import React, {
  forwardRef,
  ForwardRefRenderFunction,
  useEffect,
  useImperativeHandle,
  useState,
} from 'react';
import { TouchableOpacity, View } from 'react-native';
import RNModal from 'react-native-modal';
import { SafeAreaView } from 'react-native-safe-area-context';

import AwesomeIcon, {
  AwesomeIconVariant,
  AwesomeIconSize,
  IconName,
} from '../AwesomeIcon';
import { C, apply, classNames } from '../../styles';
import type { VisibleToggle } from '../../types';
import Color, { TextColor } from '../../Colors';
import Text, { TextVariant, FontWeight } from '../Text';
import type {
  ActionSheetItem,
  BasicActionSheetType,
} from './ActionSheet.types';

const ActionSheet: ForwardRefRenderFunction<
  VisibleToggle,
  BasicActionSheetType
> = ({ closeText, items, title, visible = false }, ref) => {
  const [isVisible, setIsVisible] = useState<boolean>(visible);
  const close = () => setIsVisible(false);
  const open = () => setIsVisible(true);

  useImperativeHandle(
    ref,
    () => ({
      open,
      close,
    }),
    []
  );

  useEffect(() => {
    setIsVisible(visible);
  }, [visible]);

  const handlePress = (item: ActionSheetItem) => () => {
    if (!item.disabled && item.onPress) {
      item.onPress();
    }
  };

  const hasIcon = items.some((item) => item.icon);

  return (
    <RNModal
      isVisible={isVisible}
      useNativeDriver
      hideModalContentWhileAnimating
      onBackButtonPress={close}
      onBackdropPress={close}
      style={apply(C.m0, C.px2, C.flex, C.justifyEnd)}
    >
      <SafeAreaView style={C.pb4}>
        <View style={apply(C.bgBackdrop, C.radius3)}>
          {title ? (
            <View style={apply(C.h10, C.itemsCenter, C.justifyCenter)}>
              <Text variant={TextVariant.XS} color={TextColor.support}>
                {title}
              </Text>
            </View>
          ) : null}

          {items?.map((item, index) => (
            <TouchableOpacity
              activeOpacity={item.onPress ? 0.8 : 1}
              onPress={handlePress(item)}
              key={index}
              style={classNames(
                'h14 row px4 itemsCenter bordertopHairline bordertopSupport',
                {
                  justifyCenter: !hasIcon,
                  justifyBetween: hasIcon,
                }
              )}
            >
              <Text
                variant={TextVariant.S}
                weight={FontWeight.Regular}
                color={TextColor.action}
              >
                {item.label}
              </Text>
              {item.icon ? (
                <AwesomeIcon
                  style={C.ml2}
                  variant={AwesomeIconVariant.Solid}
                  size={AwesomeIconSize.Medium}
                  color={Color.action}
                  icon={item.icon as IconName}
                />
              ) : null}
            </TouchableOpacity>
          ))}
        </View>
        <TouchableOpacity
          onPress={close}
          activeOpacity={0.8}
          style={apply(C.mt3, C.py4, C.bgWhite, C.radius3, C.itemsCenter)}
        >
          <Text
            variant={TextVariant.S}
            weight={FontWeight.Bold}
            color={TextColor.action}
          >
            {closeText}
          </Text>
        </TouchableOpacity>
      </SafeAreaView>
    </RNModal>
  );
};

export default forwardRef(ActionSheet);
