import { GestureResponderEvent, TouchableOpacity } from 'react-native';
import React, { useMemo, forwardRef, ForwardRefRenderFunction } from 'react';
import Text, { FontWeight, TextVariant } from './../Text';

import { ButtonProps, ButtonVariant, ButtonSize } from './Button.types';
import { classNames, C } from '../../styles';
import Color, { TextColor } from '../../Colors';
import { SocialButton } from './Button.types';
import { AppleLogo, GoogleLogo } from '../Icon';
import AwesomeIcon, {
  AwesomeIconSize,
  AwesomeIconVariant,
} from '../AwesomeIcon';

const Button: ForwardRefRenderFunction<TouchableOpacity, ButtonProps> = (
  {
    disabled,
    iconName,
    iconVariant = AwesomeIconVariant.Regular,
    onPress,
    provider,
    size = ButtonSize.Medium,
    testID,
    text,
    uppercase = false,
    variant = ButtonVariant.Primary,
  }: ButtonProps,
  ref
) => {
  const handlePress = (event: GestureResponderEvent) => {
    if (!disabled) {
      onPress(event);
    }
  };

  const textProps:
    | {
        variant: TextVariant.ExtraExtraSmall;
      }
    | {
        variant: TextVariant.Small;
        weight: FontWeight.Bold;
      } = useMemo(() => {
    switch (size) {
      case ButtonSize.Small:
        return {
          variant: TextVariant.ExtraExtraSmall,
        };

      case ButtonSize.Medium:
      case ButtonSize.Large:
      default:
        return {
          variant: TextVariant.Small,
          weight: FontWeight.Bold,
        };
    }
  }, [size]);

  const renderSocialIcon = () => {
    if (variant !== ButtonVariant.Social) {
      return null;
    }

    if (provider === SocialButton.Apple) {
      return <AppleLogo style={C.mr2} />;
    }

    return <GoogleLogo style={C.mr2} />;
  };

  const renderIcon = () => {
    if (!iconName) {
      return null;
    }

    return (
      <AwesomeIcon
        icon={iconName}
        style={C.mr2}
        variant={iconVariant}
        size={AwesomeIconSize.Medium}
        color={ButtonVariant.Primary ? Color.white : Color.squidInk}
      />
    );
  };

  return (
    <TouchableOpacity
      style={classNames('row justifyCenter px2 wFull itemsCenter', {
        'bgAction': variant === ButtonVariant.Primary,
        'bgDisrupt': variant === ButtonVariant.Secondary,
        'bgWhite': variant === ButtonVariant.Social,
        'h5 radius1': size === ButtonSize.Small,
        'h11 radius2': size === ButtonSize.Medium,
        'h13 radius2': size === ButtonSize.Large,
        'opacity50': disabled,
        uppercase,
      })}
      ref={ref}
      onPress={handlePress}
      {...{ testID }}
    >
      {renderSocialIcon()}
      {renderIcon()}
      <Text
        color={
          variant === ButtonVariant.Social
            ? TextColor.squidInk
            : TextColor.white
        }
        {...textProps}
        numberOfLines={1}
      >
        {text || ''}
      </Text>
    </TouchableOpacity>
  );
};

export default forwardRef(Button);
