import React, {
  forwardRef,
  ForwardRefRenderFunction,
  useEffect,
  useImperativeHandle,
  useMemo,
  useRef,
  useState,
} from 'react';
import {
  View,
  TextInput,
  TextInputChangeEventData,
  NativeSyntheticEvent,
  TouchableOpacity,
  Animated,
  TextInputFocusEventData,
} from 'react-native';
import { apply, C, classNames } from '../../styles';
import Color from '../../Colors';
import {
  AnimatedLabelProps,
  ForwardedInputProps,
  InputProps,
  InputSize,
} from './Input.types';
import type { IconName } from '@fortawesome/fontawesome-svg-core';
import AwesomeIcon, {
  AwesomeIconSize,
  AwesomeIconVariant,
} from '../AwesomeIcon';

const LABEL_FROM_TRANSLATE_Y = 0;
const LABEL_TO_TRANSLATE_Y = -10;
const LABEL_FROM_FONT_SIZE = 16;
const LABEL_TO_FONT_SIZE = 12;
const LABEL_FROM_LETTER_SPACING = 0.4;
const LABEL_TO_LETTER_SPACING = 0.25;
const ANIMATED_LABEL_DURATION = 400;

const AnimatedLabel = ({
  children,
  error,
  placeholderToLabel,
  style,
}: AnimatedLabelProps) => {
  const translateY = useRef(new Animated.Value(LABEL_FROM_TRANSLATE_Y)).current;
  const fontSize = useRef(new Animated.Value(LABEL_FROM_FONT_SIZE)).current;
  const letterSpacing = useRef(new Animated.Value(LABEL_FROM_LETTER_SPACING))
    .current;

  useEffect(() => {
    if (placeholderToLabel) {
      Animated.parallel([
        Animated.timing(translateY, {
          toValue: LABEL_TO_TRANSLATE_Y,
          duration: ANIMATED_LABEL_DURATION,
          useNativeDriver: false,
        }),
        Animated.timing(fontSize, {
          toValue: LABEL_TO_FONT_SIZE,
          duration: ANIMATED_LABEL_DURATION,
          useNativeDriver: false,
        }),
        Animated.timing(letterSpacing, {
          toValue: LABEL_TO_LETTER_SPACING,
          duration: ANIMATED_LABEL_DURATION,
          useNativeDriver: false,
        }),
      ]).start();
    } else {
      Animated.parallel([
        Animated.timing(translateY, {
          toValue: LABEL_FROM_TRANSLATE_Y,
          duration: ANIMATED_LABEL_DURATION,
          useNativeDriver: false,
        }),
        Animated.timing(fontSize, {
          toValue: LABEL_FROM_FONT_SIZE,
          duration: ANIMATED_LABEL_DURATION,
          useNativeDriver: false,
        }),
        Animated.timing(letterSpacing, {
          toValue: LABEL_FROM_LETTER_SPACING,
          duration: ANIMATED_LABEL_DURATION,
          useNativeDriver: false,
        }),
      ]).start();
    }
  }, [placeholderToLabel, translateY, fontSize, letterSpacing]);

  return (
    <Animated.Text
      style={apply(
        classNames('absolute wFull justifyCenter', {
          textNeutral: !error,
          textWarning: error,
        }),
        // @ts-ignore
        {
          transform: [{ translateY }],
          fontSize,
          letterSpacing,
          style,
        }
      )}
    >
      {children || ''}
    </Animated.Text>
  );
};

const Input: ForwardRefRenderFunction<ForwardedInputProps, InputProps> = (
  {
    autoCapitalize = 'none',
    cancelAnimation = false,
    containerStyle,
    disabled,
    error = false,
    focused = false,
    icon,
    inputStyle,
    onBlur,
    onChange,
    onFocus,
    placeholder,
    placeholderToLabel,
    readonly,
    required,
    secureTextEntry,
    showResetButton = false,
    size = InputSize.Medium,
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    style,
    ...props
  },
  ref
) => {
  const valueRef = useRef<string>(props.defaultValue || '');
  const inputRef = useRef<TextInput>(null);
  const [value, setValue] = useState<string>(props.defaultValue || '');
  const [isFocused, setIsFocused] = useState<boolean>(focused);
  const [isSecure, setIsSecure] = useState<boolean>(secureTextEntry ?? false);
  const [showReset, setShowReset] = useState<boolean>(false);

  useEffect(() => {
    setIsFocused(focused);
  }, [focused]);

  useImperativeHandle(
    ref,
    () => ({
      getValue: () => valueRef.current,
      setValue: (val: string) => (valueRef.current = val),
    }),
    []
  );

  const handleChange = (e: NativeSyntheticEvent<TextInputChangeEventData>) => {
    valueRef.current = e.nativeEvent.text;
    setValue(e.nativeEvent.text);
    onChange?.(e.nativeEvent.text);
    setShowReset(valueRef.current !== '');
  };

  const handleFocus = (e: NativeSyntheticEvent<TextInputFocusEventData>) => {
    setIsFocused(true);
    onFocus?.(e);
  };

  const handleBlur = (e: NativeSyntheticEvent<TextInputFocusEventData>) => {
    setIsFocused(false);
    onBlur?.(e);
  };

  const animatePlaceholder = useMemo(() => {
    if (placeholderToLabel) {
      return placeholderToLabel(isFocused, value);
    }

    return isFocused || value.length > 0;
  }, [isFocused, placeholderToLabel, value]);

  const toggleSecure = () => setIsSecure((prev) => !prev);
  const resetInput = () => {
    valueRef.current = '';
    setValue('');
    onChange?.('');
    setShowReset(false);
    inputRef.current?.clear();
  };

  return (
    <View
      style={apply(
        classNames(
          'row wFull borderMinimum borderPlatin radius2 itemsCenter px4',
          {
            borderWarning: error,
            borderAction: isFocused,
            borderTransparent: disabled || readonly,
            h14: size === InputSize.Medium,
            h12: size === InputSize.Small,
            bgPlatin: !disabled,
            bgDisabled: disabled,
            bgReadonly: readonly,
          }
        ),
        containerStyle
      )}
    >
      {icon && !readonly ? (
        <AwesomeIcon
          color={
            disabled
              ? Color.neutral
              : !isFocused && error
              ? Color.warning
              : Color.action
          }
          icon={
            !isFocused && error ? 'exclamation-triangle' : (icon as IconName)
          }
          size={AwesomeIconSize.ExtraSmall}
          style={C.mr4}
          variant={AwesomeIconVariant.Regular}
        />
      ) : null}
      <View style={apply(C.hFull, C.justifyCenter, C.flex)}>
        <AnimatedLabel
          error={!isFocused && error}
          placeholderToLabel={
            !disabled && !cancelAnimation && animatePlaceholder
          }
        >
          {`${placeholder} ${required ? '*' : ''}`}
        </AnimatedLabel>
        {(disabled || readonly) && (
          <View style={apply(C.hFull, C.wFull, C.absolute, { zIndex: 2 })} />
        )}
        <TextInput
          autoCapitalize={autoCapitalize}
          defaultValue={valueRef.current}
          editable={disabled}
          onBlur={handleBlur}
          onChange={handleChange}
          onFocus={handleFocus}
          ref={inputRef}
          secureTextEntry={secureTextEntry && isSecure}
          style={apply(
            C.hFull,
            C.flex,
            C.font4,
            C.p0,
            C.pt4,
            C.textSquidInk,
            inputStyle
          )}
          underlineColorAndroid={Color.transparent}
          {...props}
        />
      </View>
      {showReset && showResetButton ? (
        <TouchableOpacity activeOpacity={0.8} onPress={resetInput}>
          <AwesomeIcon
            color={Color.neutral}
            icon="times"
            size={AwesomeIconSize.ExtraSmall}
            style={C.ml2}
            variant={AwesomeIconVariant.Light}
          />
        </TouchableOpacity>
      ) : null}
      {secureTextEntry ? (
        <TouchableOpacity activeOpacity={0.8} onPress={toggleSecure}>
          <AwesomeIcon
            color={Color.action}
            icon={isSecure ? 'eye' : 'eye-slash'}
            size={AwesomeIconSize.ExtraSmall}
            style={C.ml2}
            variant={AwesomeIconVariant.Regular}
          />
        </TouchableOpacity>
      ) : null}
    </View>
  );
};

export default forwardRef(Input);
