import React, {
  forwardRef,
  ForwardRefRenderFunction,
  useEffect,
  useImperativeHandle,
  useState,
} from 'react';
import {
  Platform,
  KeyboardAvoidingView,
  ScrollView,
  TouchableOpacity,
  View,
} from 'react-native';
import RNModal from 'react-native-modal';

import type { ModalProps } from './Modal.types';
import Text, { FontWeight, TextVariant } from '../Text';
import type { VisibleToggle } from '../../types';
import Color from '../../Colors';
import AwesomeIcon, {
  AwesomeIconSize,
  AwesomeIconVariant,
} from '../AwesomeIcon';
import { apply, C } from '../../styles';

const Modal: ForwardRefRenderFunction<VisibleToggle, Partial<ModalProps>> = (
  { title = '', footer, children, isVisible = false, ...props },
  ref
) => {
  const [visible, setVisible] = useState<boolean>(isVisible);
  const close = () => setVisible(false);
  const open = () => setVisible(true);

  useImperativeHandle(
    ref,
    () => ({
      open,
      close,
    }),
    []
  );

  useEffect(() => {
    setVisible(visible);
  }, [isVisible, visible]);
  return (
    <RNModal isVisible={visible} style={apply(C.flex, C.m0)} {...props}>
      <View
        style={apply(C.flex, C.bgWhite, {
          borderTopLeftRadius: 12,
          borderTopRightRadius: 12,
        })}
      >
        <View
          style={apply(
            C.row,
            C.justifyBetween,
            C.itemsCenter,
            C.mt3,
            C.py3,
            C.px4,
            C.mb1
          )}
        >
          <Text
            variant={TextVariant.Large}
            weight={FontWeight.Bold}
            numberOfLines={1}
          >
            {title}
          </Text>
          <TouchableOpacity onPress={close} style={C.ml2}>
            <AwesomeIcon
              variant={AwesomeIconVariant.Regular}
              icon="times"
              size={AwesomeIconSize.Medium}
              color={Color.support}
            />
          </TouchableOpacity>
        </View>
        {Platform.OS === 'ios' ? (
          <KeyboardAvoidingView
            style={C.flex}
            keyboardVerticalOffset={50}
            behavior={'padding'}
          >
            <ScrollView style={apply(C.flex)}>{children}</ScrollView>
          </KeyboardAvoidingView>
        ) : (
          <ScrollView style={apply(C.flex)}>{children}</ScrollView>
        )}
        {footer}
      </View>
    </RNModal>
  );
};

export default forwardRef(Modal);
