import React, {
  forwardRef,
  ForwardRefRenderFunction,
  useEffect,
  useImperativeHandle,
  useMemo,
  useRef,
  useState,
} from 'react';
import { Platform, View, Text, TouchableOpacity } from 'react-native';
import RNPickerSelect from 'react-native-picker-select';
import Color from '../../Colors';
import { apply, C, classNames } from '../../styles';
import AwesomeIcon, {
  AwesomeIconSize,
  AwesomeIconVariant,
  IconName,
} from '../AwesomeIcon';
import type { SelectProps, ForwardedSelectProps } from './Select.types';

const Select: ForwardRefRenderFunction<ForwardedSelectProps, SelectProps> = (
  {
    containerStyle,
    disabled = false,
    error = false,
    focused = false,
    icon,
    inputStyle,
    items,
    label,
    onChange,
    placeholder,
    readonly = false,
    value,
  },
  ref
) => {
  const [currentValue, setCurrentValue] = useState<string | number | null>(
    value || null
  );
  const selectRef = useRef<RNPickerSelect>(null);
  const [isFocused, setIsFocused] = useState<boolean>(focused);

  const handleChange = (val: string | number | null) => {
    setCurrentValue(val);
    onChange?.(val);
    setIsFocused(false);
  };

  useImperativeHandle(
    ref,
    () => ({
      getValue: () => currentValue,
      setValue: (val: string | number | null) => setCurrentValue(val),
    }),
    [currentValue]
  );

  const handleOpen = () => {
    if (Platform.OS === 'ios') {
      setIsFocused(true);
    }
  };

  const handleClose = () => {
    if (Platform.OS === 'ios') {
      setIsFocused(false);
    }
  };

  const canInteract = useMemo(() => !disabled && !readonly, [
    disabled,
    readonly,
  ]);

  const handlePress = () => {
    if (canInteract) {
      selectRef.current?.togglePicker();
    }
  };

  const innerPlaceholder = useMemo(
    () =>
      ({
        label: placeholder ?? label,
        value: null,
      } || null),
    [placeholder, label]
  );

  useEffect(() => {
    setIsFocused(focused);
  }, [focused]);

  useEffect(() => {
    setCurrentValue(value ?? null);
  }, [value]);

  return (
    <TouchableOpacity
      activeOpacity={1}
      onPress={handlePress}
      style={apply(
        classNames(
          'row wFull h14 borderMinimum borderPlatin radius2 itemsCenter px4',
          {
            borderWarning: error,
            borderAction: isFocused,
            borderTransparent: disabled || readonly,
            bgPlatin: !disabled,
            bgDisabled: disabled,
            bgReadonly: readonly,
          }
        ),
        containerStyle
      )}
    >
      {icon && !readonly ? (
        <AwesomeIcon
          color={
            disabled
              ? Color.neutral
              : !isFocused && error
              ? Color.warning
              : Color.action
          }
          icon={
            !isFocused && error ? 'exclamation-triangle' : (icon as IconName)
          }
          size={AwesomeIconSize.ExtraSmall}
          style={C.mr4}
          variant={AwesomeIconVariant.Regular}
        />
      ) : null}
      <View style={apply(C.hFull, C.justifyCenter, C.flex)}>
        <Text
          style={apply(
            classNames('absolute wFull justifyCenter top2', {
              textWarning: !isFocused && error,
              textAction: isFocused,
              textNeutral: !error,
            }),
            {
              fontSize: 12,
            }
          )}
        >
          {label || placeholder}
        </Text>
        <RNPickerSelect
          disabled={!canInteract}
          onValueChange={handleChange}
          onOpen={handleOpen}
          onClose={handleClose}
          items={items}
          touchableWrapperProps={{
            style: classNames('hFull justifyCenter pt4', {}),
          }}
          useNativeAndroidPickerStyle={false}
          textInputProps={{
            // @ts-ignore
            style: apply(classNames('font4 hFull p0 textSquidInk'), inputStyle),
          }}
          ref={selectRef}
          Icon={() => null}
          fixAndroidTouchableBug={true}
          placeholder={innerPlaceholder}
          value={currentValue}
        />
      </View>
      {canInteract ? (
        <TouchableOpacity onPress={handlePress}>
          <AwesomeIcon
            color={Color.action}
            icon="chevron-down"
            size={AwesomeIconSize.ExtraSmall}
            style={C.ml2}
            variant={AwesomeIconVariant.Regular}
          />
        </TouchableOpacity>
      ) : null}
    </TouchableOpacity>
  );
};

export default forwardRef(Select);
